﻿Class Window1 
    Private Const offset_a As Integer = 3
    Private Const offset_r As Integer = 2
    Private Const offset_g As Integer = 1
    Private Const offset_b As Integer = 0

    ' Return the ARGB components of a pixel.
    Private Sub GetPixel(ByVal pixels() As Byte, ByVal stride As Integer, ByVal bytes_per_pixel As Integer, ByVal x As Integer, ByVal y As Integer, ByRef a As Byte, ByRef r As Byte, ByRef g As Byte, ByRef b As Byte)
        Dim offset As Integer = y * stride + x * bytes_per_pixel
        a = pixels(offset + offset_a)
        r = pixels(offset + offset_r)
        g = pixels(offset + offset_g)
        b = pixels(offset + offset_b)
    End Sub

    ' Set the ARGB components of a pixel.
    Private Sub SetPixel(ByVal pixels() As Byte, ByVal stride As Integer, ByVal bytes_per_pixel As Integer, ByVal x As Integer, ByVal y As Integer, ByVal a As Byte, ByVal r As Byte, ByVal g As Byte, ByVal b As Byte)
        Dim offset As Integer = y * stride + x * bytes_per_pixel
        pixels(offset + offset_a) = a
        pixels(offset + offset_r) = r
        pixels(offset + offset_g) = g
        pixels(offset + offset_b) = b
    End Sub

    ' Invert the original image.
    Private Sub btnInvert_Click(ByVal sender As System.Object, ByVal e As System.Windows.RoutedEventArgs) Handles btnInvert.Click
        ' Get the bitmap source.
        Dim bms_old As BitmapSource = imgOld.Source

        ' Get the size in pixels.
        Dim wid As Integer = bms_old.PixelWidth
        Dim hgt As Integer = bms_old.PixelHeight

        ' Make room for the pixel bytes.
        Dim bytes_per_pixel As Integer = bms_old.Format.BitsPerPixel \ 8
        Dim stride As Integer = wid * bytes_per_pixel
        Dim pixels_old(hgt * stride - 1) As Byte

        ' Get the pixel data.
        Try
            bms_old.CopyPixels(pixels_old, stride, 0)
        Catch ex As Exception
            MessageBox.Show(ex.Message)
            Exit Sub
        End Try

        ' Make room for the output pixel data.
        Dim pixels_new(hgt * stride - 1) As Byte

        ' Transform the pixel data.
        pixels_old.CopyTo(pixels_new, 0)
        For x As Integer = 0 To wid - 1
            For y As Integer = 0 To hgt - 1
                Dim a As Byte = 0
                Dim r As Byte = 0
                Dim g As Byte = 0
                Dim b As Byte = 0
                GetPixel(pixels_old, stride, bytes_per_pixel, _
                    x, y, a, r, g, b)

                ' Invert.
                r = 255 - r
                g = 255 - g
                b = 255 - b
                SetPixel(pixels_new, stride, bytes_per_pixel, _
                    x, y, a, r, g, b)
            Next y
        Next x

        ' Display the result.
        Dim bms_new As BitmapSource = BitmapSource.Create(wid, hgt, _
            bms_old.DpiX, bms_old.DpiY, bms_old.Format, _
            Nothing, pixels_new, stride)
        imgNew.Source = bms_new
    End Sub

    ' Keep only the red color component.
    Private Sub btnRed_Click(ByVal sender As System.Object, ByVal e As System.Windows.RoutedEventArgs)
        CopyColors(True, True, False, False)
    End Sub

    ' Keep only the green color component.
    Private Sub btnGreen_Click(ByVal sender As System.Object, ByVal e As System.Windows.RoutedEventArgs)
        CopyColors(True, False, True, False)
    End Sub

    ' Keep only the blue color component.
    Private Sub btnBlue_Click(ByVal btn As Button, ByVal a As Object) Handles btnBlue.Click
        ' MessageBox.Show(btn.Name)
        CopyColors(True, False, False, True)
    End Sub

    ' Copy red, green, or blue color components.
    Private Sub CopyColors(ByVal copy_a As Boolean, ByVal copy_r As Boolean, ByVal copy_g As Boolean, ByVal copy_b As Boolean)
        ' Get the bitmap source.
        Dim bms_old As BitmapSource = imgOld.Source

        ' Get the size in pixels.
        Dim wid As Integer = bms_old.PixelWidth
        Dim hgt As Integer = bms_old.PixelHeight

        ' Make room for the pixel bytes.
        Dim bytes_per_pixel As Integer = bms_old.Format.BitsPerPixel \ 8
        Dim stride As Integer = wid * bytes_per_pixel
        Dim pixels_old(hgt * stride - 1) As Byte

        ' Get the pixel data.
        Try
            bms_old.CopyPixels(pixels_old, stride, 0)
        Catch ex As Exception
            MessageBox.Show(ex.Message)
            Exit Sub
        End Try

        ' Make room for the output pixel data.
        Dim pixels_new(hgt * stride - 1) As Byte

        ' Transform the pixel data.
        pixels_old.CopyTo(pixels_new, 0)
        For x As Integer = 0 To wid - 1
            For y As Integer = 0 To hgt - 1
                Dim a As Byte = 0
                Dim r As Byte = 0
                Dim g As Byte = 0
                Dim b As Byte = 0
                GetPixel(pixels_old, stride, bytes_per_pixel, _
                    x, y, a, r, g, b)

                ' Save only the selected components.
                If (Not copy_a) Then a = 0
                If (Not copy_r) Then r = 0
                If (Not copy_g) Then g = 0
                If (Not copy_b) Then b = 0

                SetPixel(pixels_new, stride, bytes_per_pixel, _
                    x, y, a, r, g, b)
            Next y
        Next x

        ' Display the result.
        Dim bms_new As BitmapSource = BitmapSource.Create(wid, hgt, _
            bms_old.DpiX, bms_old.DpiY, bms_old.Format, _
            Nothing, pixels_new, stride)
        imgNew.Source = bms_new
    End Sub

    ' Convert to grayscale.
    Private Sub btnGrayscale_Click() Handles btnGrayscale.Click
        ' Get the bitmap source.
        Dim bms_old As BitmapSource = imgOld.Source

        ' Get the size in pixels.
        Dim wid As Integer = bms_old.PixelWidth
        Dim hgt As Integer = bms_old.PixelHeight

        ' Make room for the pixel bytes.
        Dim bytes_per_pixel As Integer = bms_old.Format.BitsPerPixel \ 8
        Dim stride As Integer = wid * bytes_per_pixel
        Dim pixels_old(hgt * stride - 1) As Byte

        ' Get the pixel data.
        Try
            bms_old.CopyPixels(pixels_old, stride, 0)
        Catch ex As Exception
            MessageBox.Show(ex.Message)
            Exit Sub
        End Try

        ' Make room for the output pixel data.
        Dim pixels_new(hgt * stride - 1) As Byte

        ' Transform the pixel data.
        pixels_old.CopyTo(pixels_new, 0)
        For x As Integer = 0 To wid - 1
            For y As Integer = 0 To hgt - 1
                Dim a As Byte = 0
                Dim r As Byte = 0
                Dim g As Byte = 0
                Dim b As Byte = 0
                GetPixel(pixels_old, stride, bytes_per_pixel, _
                    x, y, a, r, g, b)

                ' Convert to gray scale.
                Dim gray As Byte = CByte(0.3 * r + 0.5 * g + 0.2 * b)
                SetPixel(pixels_new, stride, bytes_per_pixel, _
                    x, y, a, gray, gray, gray)
            Next y
        Next x

        ' Display the result.
        Dim bms_new As BitmapSource = BitmapSource.Create(wid, hgt, _
            bms_old.DpiX, bms_old.DpiY, bms_old.Format, _
            Nothing, pixels_new, stride)
        imgNew.Source = bms_new
    End Sub

    ' Average the red, green, and blue components.
    Private Sub btnAverage_Click(ByVal sender As System.Object, ByVal e As System.Windows.RoutedEventArgs)
        ' Get the bitmap source.
        Dim bms_old As BitmapSource = imgOld.Source

        ' Get the size in pixels.
        Dim wid As Integer = bms_old.PixelWidth
        Dim hgt As Integer = bms_old.PixelHeight

        ' Make room for the pixel bytes.
        Dim bytes_per_pixel As Integer = bms_old.Format.BitsPerPixel \ 8
        Dim stride As Integer = wid * bytes_per_pixel
        Dim pixels_old(hgt * stride - 1) As Byte

        ' Get the pixel data.
        Try
            bms_old.CopyPixels(pixels_old, stride, 0)
        Catch ex As Exception
            MessageBox.Show(ex.Message)
            Exit Sub
        End Try

        ' Make room for the output pixel data.
        Dim pixels_new(hgt * stride - 1) As Byte

        ' Transform the pixel data.
        pixels_old.CopyTo(pixels_new, 0)
        For x As Integer = 0 To wid - 1
            For y As Integer = 0 To hgt - 1
                Dim a As Byte = 0
                Dim r As Byte = 0
                Dim g As Byte = 0
                Dim b As Byte = 0
                GetPixel(pixels_old, stride, bytes_per_pixel, _
                    x, y, a, r, g, b)

                ' Average the components.
                Dim gray As Byte = CByte((CInt(r) + g + b) / 3)
                SetPixel(pixels_new, stride, bytes_per_pixel, _
                    x, y, a, gray, gray, gray)
            Next y
        Next x

        ' Display the result.
        Dim bms_new As BitmapSource = BitmapSource.Create(wid, hgt, _
            bms_old.DpiX, bms_old.DpiY, bms_old.Format, _
            Nothing, pixels_new, stride)
        imgNew.Source = bms_new
    End Sub

    ' Connect btnRed to its event handler.
    Private Sub Window1_Loaded(ByVal sender As System.Object, ByVal e As System.Windows.RoutedEventArgs) Handles MyBase.Loaded
        AddHandler btnRed.Click, AddressOf btnRed_Click
    End Sub
End Class
